/* ============================================================================
 * Copyright (c) 1998 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Every modification must be notified to the Java Apache Project
 *    and redistribution of the modified code without prior notification
 *    is NOT permitted in any form.
 *
 * 4. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache Project
 *    (http://java.apache.org/)."
 *
 * 5. The names "Apache JMeter" and "Java Apache Project" must not be used to 
 *    endorse or promote products derived from this software without prior 
 *    written permission.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache Project
 *    (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software was developed by Beta Version Productions and 
 * donated to the Java Apache Project that continues its development.
 * For more information on the Java Apache Project and the JMeter project, 
 * please see <http://java.apache.org/>.
 */

package org.apache.jmeter;

import java.io.*;
import java.awt.*;
import java.net.*;
import java.util.*;
import java.awt.event.*;
import org.apache.jmeter.timers.*;
import org.apache.jmeter.visualisers.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

/**
 * This is Apache JMeter main class that implements both drawing and action 
 * listening.
 *
 * @author  Stefano Mazzocchi <mazzocch@mbox.systemy.it>
 * @version $Revision: 1.3 $ $Date: 1999/01/25 15:41:41 $
 */
public class JMeter extends JFrame implements ActionListener, ChangeListener {

    public static final String VERSION = "Apache JMeter 1.0.2";
    public static final String PROPERTIES = "org/apache/jmeter/jmeter.properties";
    public static final String IMAGES = "org/apache/jmeter/images/";
    public static final int MAX_THREADS = 10;
    public static final int START_THREADS = 1;

    int samplers = START_THREADS;
    Vector sampler;
    Visualiser visualiser;
    Properties properties;
    org.apache.jmeter.timers.Timer timer;
    
    JButton start;
    JButton stop;
    JButton clear;
    JSlider slider;
    JTextField urlTextField;
    JTextField queryTextField;
    JCheckBox queryCheck;
    JComboBox timerList;
    JComboBox visualiserList;
    JDialog about;
    JPanel timers;
    JPanel visualisers;
    ImageIcon disabled;
    ImageIcon idle;
    
    JMeter(String file) {
        super(VERSION);

        this.properties = getProperties(file);
        this.sampler = new Vector();
        
        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }  
        });
        
        this.setLocation(100, 100);
        this.draw();
        this.pack();
        this.setVisible(true);
    }

    /**
     * This method is used by the constructor to load the property
     * file that may even reside in the user space, or in the 
     * classpath under org.apache.jmeter.jmeter.properties
     */
    Properties getProperties(String file) {
        
        Properties p = new Properties();
        
        try {
            File f = new File (System.getProperty("user.dir") + File.separator + file);
            p.load(new FileInputStream(f));
        } catch (Exception e) {
            p.clear();
            try {
                p.load(this.getClass().getClassLoader().getSystemResourceAsStream(PROPERTIES));
            } catch (IOException ex) {
                JMeter.fail("Could not read internal resource. Archive is broken.");
            }
        }
        
        return p;
    }
    
    /**
     * This looks for the requested image in the classpath
     * under org.apache.jmeter.images.<name>
     */
    ImageIcon getImage(String name) {
        return new ImageIcon(this.getClass().getClassLoader().getSystemResource(IMAGES + name));
    }

    /**
     * Creates the vector of timer plugins.
     */
    Vector getTimers() {
        Vector v = new Vector();
        Enumeration names = properties.keys();
        while (names.hasMoreElements()) {
            String prop = (String) names.nextElement();
            if ((prop.startsWith("timer.")) && (Boolean.valueOf(properties.getProperty(prop)).booleanValue())) {
                String name = prop.substring(6, prop.length());
                try {
                    v.addElement((org.apache.jmeter.timers.Timer) Class.forName(name).newInstance());
                } catch (ClassNotFoundException e) {
                    JMeter.fail("Error loading class " + name + ": class is not found");
                } catch (IllegalAccessException e) {
                    JMeter.fail("Error loading class " + name + ": does not have access");
                } catch (InstantiationException e) {
                    JMeter.fail("Error loading class " + name + ": could not instantiate");
                } catch (ClassCastException e) {
                    JMeter.fail("Error loading class " + name + ": it's not a Timer");
                }
            }
        }
        return v;
    }
        
    /**
     * Creates the vector of visualiser plugins.
     */
    Vector getVisualisers() {
        Vector v = new Vector();
        Enumeration names = properties.keys();
        while (names.hasMoreElements()) {
            String prop = (String) names.nextElement();
            if ((prop.startsWith("visualiser.")) && (Boolean.valueOf(properties.getProperty(prop)).booleanValue())) {
                String name = prop.substring(11, prop.length());
                try {
                    v.addElement((Visualiser) Class.forName(name).newInstance());
                } catch (ClassNotFoundException e) {
                    JMeter.fail("Error loading class " + name + ": class is not found");
                } catch (IllegalAccessException e) {
                    JMeter.fail("Error loading class " + name + ": does not have access");
                } catch (InstantiationException e) {
                    JMeter.fail("Error loading class " + name + ": could not instantiate");
                } catch (ClassCastException e) {
                    JMeter.fail("Error loading class " + name + ": it's not a Timer");
                }
            }
        }
        return v;
    }
    
    /**
     * Create the Netscape style button
     */
    JButton createButton(String name) {
        JButton button = new JButton(getImage(name + ".on.gif"));
        button.setDisabledIcon(getImage(name + ".off.gif"));
        button.setRolloverIcon(getImage(name + ".over.gif"));
        button.setPressedIcon(getImage(name + ".down.gif"));
        button.setActionCommand(name);
        button.addActionListener(this);
        button.setRolloverEnabled(true);
        button.setFocusPainted(false);
        button.setBorderPainted(false);
        button.setOpaque(false);
        button.setPreferredSize(new Dimension(40, 40));
        return button;
    }
        
    /**
     * Draw the whole thing.
     */
    void draw() {

        // Draw toolbar
        start = createButton("start");
        stop = createButton("stop");
        stop.setEnabled(false);
        JButton clear = createButton("clear");
        JButton about = createButton("about");

        JPanel leftButtons = new JPanel();
        leftButtons.setLayout(new FlowLayout(FlowLayout.LEFT));
        leftButtons.add(start);
        leftButtons.add(clear);
        leftButtons.add(stop);
        
        JPanel rightButtons = new JPanel();
        rightButtons.setLayout(new FlowLayout(FlowLayout.RIGHT));
        rightButtons.add(about);
        
        JPanel toolBar = new JPanel();        
        toolBar.setBorder(new BevelBorder(BevelBorder.RAISED));
        toolBar.setLayout(new BorderLayout());
        toolBar.add(leftButtons, BorderLayout.WEST);
        toolBar.add(rightButtons, BorderLayout.EAST);

        // Draw URL Panel        
        Border etched = new EtchedBorder();
        JPanel urlPanel = new JPanel();
        urlPanel.setBorder(new TitledBorder(etched, "URL"));
        GridBagLayout g = new GridBagLayout();
        urlPanel.setLayout(g);
        GridBagConstraints c = new GridBagConstraints();
        c.fill = c.BOTH;
        
        JLabel urlLabel = new JLabel("URL:", JLabel.RIGHT);
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.weightx = 0.0;
        c.weighty = 0.0;
        g.setConstraints(urlLabel, c);
        urlPanel.add(urlLabel);
        
        urlTextField = new JTextField("http://localhost");
        c.gridx = 1;
        c.weightx = 1.0;
        c.insets = new Insets(3, 3, 3, 3);
        g.setConstraints(urlTextField, c);
        urlPanel.add(urlTextField);
        
        queryCheck = new JCheckBox("Post:");
        queryCheck.setFocusPainted(false);
        queryCheck.setForeground(urlLabel.getForeground());
        queryCheck.setActionCommand("post");
        queryCheck.addActionListener(this);
        c.gridx = 0;
        c.gridy = 1;
        c.weightx = 0.0;
        g.setConstraints(queryCheck, c);
        urlPanel.add(queryCheck);

        queryTextField = new JTextField();
        queryTextField.setEnabled(false);
        queryTextField.setBackground(this.getBackground());
        c.gridx = 1;
        c.weightx = 1.0;
        c.insets = new Insets(3, 3, 3, 3);
        g.setConstraints(queryTextField, c);
        urlPanel.add(queryTextField);

        // Draw Thread Panel
        JPanel threadPanel = new JPanel();
        threadPanel.setBorder(new TitledBorder(etched, "Threads"));
        threadPanel.setLayout(new BorderLayout());
    
        slider = new JSlider(JSlider.HORIZONTAL, 1, MAX_THREADS, samplers);
        slider.addChangeListener(this);
        slider.setMajorTickSpacing(1);
        slider.setMinorTickSpacing(1);
        slider.setPaintTicks(true);
        slider.setPaintLabels(true);
        g.setConstraints(slider, c);
        threadPanel.add(slider, BorderLayout.NORTH);

        JPanel ledPanel = new JPanel();
        ledPanel.setLayout(new FlowLayout());
        ledPanel.setBorder(new BevelBorder(BevelBorder.LOWERED));

        disabled = getImage("thread.disabled.gif");
        idle = getImage("thread.idle.gif");
        
        for (int i = 0; i < MAX_THREADS; i++) {
            URLSampler s = new URLSampler(getImage("thread.enabled.gif"));
            s.setDisabledIcon(disabled);
            s.setEnabled(false);
            sampler.addElement(s);
            ledPanel.add((JLabel) s);
        }

        threadPanel.add(ledPanel, BorderLayout.CENTER);
        
        // Draw Timer Panel
        JPanel timerPanel = new JPanel();
        timerPanel.setBorder(new TitledBorder(etched, "Timers"));
        timerPanel.setLayout(new BorderLayout());
        
        Vector t = getTimers();
        timerList = new JComboBox(t);
        timerList.setEditable(false);
        timerList.setSelectedIndex(0);
        timerList.addActionListener(this);
        timerList.setActionCommand("timer");
        timerPanel.add(timerList, BorderLayout.NORTH);
        
        timers = new JPanel();
        timers.setLayout(new CardLayout());
        for (Enumeration e = t.elements(); e.hasMoreElements();) {
            JComponent timer = (JComponent) e.nextElement();
            timers.add(timer, timer.toString());
        }
        timerPanel.add(timers, BorderLayout.CENTER);
        
        // Draw Visualiser Panel
        JPanel visualiserPanel = new JPanel();
        visualiserPanel.setBorder(new TitledBorder(etched, "Visualiser"));
        visualiserPanel.setLayout(new BorderLayout());
        
        Vector a = getVisualisers();
        visualiserList = new JComboBox(a);
        visualiserList.setEditable(false);
        visualiserList.setSelectedIndex(0);
        visualiserList.addActionListener(this);
        visualiserList.setActionCommand("visualiser");
        visualiserPanel.add(visualiserList, BorderLayout.NORTH);
        
        visualisers = new JPanel();
        visualisers.setLayout(new CardLayout());
        for (Enumeration e = a.elements(); e.hasMoreElements();) {
            JComponent visualiser = (JComponent) e.nextElement();
            visualisers.add(visualiser, visualiser.toString());
        }
        visualiserPanel.add(visualisers, BorderLayout.CENTER);

        // Add all panels to main layout    
        JPanel mainPanel = new JPanel();
        g = new GridBagLayout();
        c = new GridBagConstraints();
        c.fill = c.BOTH;
        mainPanel.setLayout(g);
        mainPanel.setBorder(new EmptyBorder(0, 5, 5, 5));
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 2;
        c.gridheight = 1;
        c.weightx = 1.0;
        c.weighty = 0.0;
        g.setConstraints(urlPanel, c);
        mainPanel.add(urlPanel);
        c.gridy = 1;
        c.gridwidth = 1;
        c.weightx = 0.2;
        g.setConstraints(threadPanel, c);
        mainPanel.add(threadPanel);
        c.gridx = 1;
        c.gridy = 1;
        c.gridwidth = 1;
        c.weightx = 0.8;
        g.setConstraints(timerPanel, c);
        mainPanel.add(timerPanel);
        c.gridx = 0;
        c.gridy = 2;
        c.gridwidth = 2;
        c.weighty = 1.0;
        g.setConstraints(visualiserPanel, c);
        mainPanel.add(visualiserPanel);
        
        // Set main panels
        this.getContentPane().add(toolBar, BorderLayout.NORTH);
        this.getContentPane().add(mainPanel, BorderLayout.CENTER);
    }

    /**
     * Action listener that handles all GUI events in this class.
     */
    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand().equals("start")) {
            start();
            repaint();
        } else if (e.getActionCommand().equals("stop")) {
            stop();
            repaint();
        } else if (e.getActionCommand().equals("clear")) {
            this.visualiser.clear();
            repaint();
        } else if (e.getActionCommand().equals("timer")) {
            this.timer = (org.apache.jmeter.timers.Timer) timerList.getSelectedItem();
            ((CardLayout) timers.getLayout()).show(timers,(String) this.timer.toString());            
            repaint();
        } else if (e.getActionCommand().equals("visualiser")) {
            this.visualiser = (Visualiser) visualiserList.getSelectedItem();
            ((CardLayout) visualisers.getLayout()).show(visualisers,(String) this.visualiser.toString());
            for (int i = 0; i < samplers; i++) {
                ((URLSampler) sampler.elementAt(i)).set(this.visualiser);
            }    
            repaint();
        } else if (e.getActionCommand().equals("post")) {
            if (queryCheck.isSelected()) {
                queryTextField.setBackground(urlTextField.getBackground());
                queryTextField.setEnabled(true);
            } else {
                queryTextField.setBackground(this.getBackground());
                queryTextField.setEnabled(false);
            }
            repaint();
        } else if (e.getActionCommand().equals("about")) {
            about();
        } else {
            // do nothing!
        }
    }
    
    /**
     * Change listener that handles thread slider events.
     */
    public void stateChanged(ChangeEvent e) {
        if (!this.slider.getValueIsAdjusting()) {
            int i = 0;
            
            this.samplers = this.slider.getValue();
            
            while (i < samplers) {
                URLSampler s = (URLSampler) sampler.elementAt(i++);
                s.setDisabledIcon(idle);
            }
            
            while (i < MAX_THREADS) {
                URLSampler s = (URLSampler) sampler.elementAt(i++);
                s.setDisabledIcon(disabled);
            }
            
            repaint();
        }
    }
    
    /**
     * Called by start button. Starts up sampling threads.
     */
    void start() {
            
        stop.setEnabled(true);
        start.setEnabled(false);
        slider.setEnabled(false);
        urlTextField.setEnabled(false);
        timerList.setEnabled(false);
        queryCheck.setEnabled(false);
        
        try {
            String query = (queryCheck.isSelected()) ? URLEncoder.encode(queryTextField.getText()) : null;
            URL url = URLSampler.tryURL(urlTextField.getText(), query);
            this.samplers = slider.getValue();
            this.timer = (org.apache.jmeter.timers.Timer) timerList.getSelectedItem();
            this.visualiser = (Visualiser) visualiserList.getSelectedItem();
            
            for (int i = 0; i < samplers; i++) {
                URLSampler s = (URLSampler) sampler.elementAt(i);
                s.set(visualiser, timer, url, query);
                new Thread(s).start();
            }    
        } catch (Exception e) {
            JMeter.error(e, this);
            stop();
        }
    }
    
    /**
     * Called by stop button. Stops sampling threads.
     */
    void stop() {
        
        stop.setEnabled(false);
        start.setEnabled(true);
        slider.setEnabled(true);
        urlTextField.setEnabled(true);
        timerList.setEnabled(true);
        queryCheck.setEnabled(true);
        
        // NOTE: I do this instead of calling Thread.stop() because
        // that method is deprecated and there is no valid alternative
        // Calling the die() method does not stop the thread but
        // blocks the thread when it's done its current sampling
        // process and avoids last sample to get collected.
        for (int i = 0; i < samplers; i++) {
            ((URLSampler) sampler.elementAt(i)).die();
        }    
    }

    /**
     * Called by about button. Raises about dialog.
     */
    void about() {
        if (about == null) {
            about = new JDialog(this, "About Apache JMeter...", true);
            
            JLabel jmeter = new JLabel(this.getImage("jmeter.jpg"));
            
            JLabel copyright = new JLabel("Copyright (c) 1998 The Java Apache Project", JLabel.CENTER);
            JLabel rights = new JLabel("All Rights Reserved.", JLabel.CENTER);
            JPanel infos = new JPanel();
            infos.setOpaque(false);
            infos.setLayout(new GridLayout(0, 1));
            infos.setBorder(new EmptyBorder(5, 5, 5, 5));
            infos.add(copyright);
            infos.add(rights);
            
            Container panel = about.getContentPane();
            panel.setLayout(new BorderLayout());
            panel.setBackground(Color.white);
            panel.add(jmeter, BorderLayout.NORTH);
            panel.add(infos, BorderLayout.SOUTH);
        }
        
        // NOTE: these lines center the about dialog in the
        // current window. Some older Swing versions have
        // a bug in getLocationOnScreen() and they may not
        // make this behave properly.
        Point p = this.getLocationOnScreen();
        Dimension d1 = this.getSize();
        Dimension d2 = about.getSize();
        about.setLocation(p.x + (d1.width - d2.width) / 2, 
            p.y + (d1.height - d2.height) / 2);
        about.pack();
        about.setVisible(true);
    }

    /**
     * Main method that chooses L&F and created the JMeter object.
     */
    public static void main(String arguments[]) {

        try {
            UIManager.setLookAndFeel(
               UIManager.getCrossPlatformLookAndFeelClassName());
        } catch (Exception e) {
            System.err.println("Couldn't use the Java "
                + "look and feel: " + e);
        }

        if (arguments.length > 1) {
            new JMeter(arguments[0]);
        } else {
            new JMeter(null);
        }
    }
    
    /**
     * Method used to signal an Exception to the user.
     */
    public static void error(Exception e, Component thrower) {
        JOptionPane.showMessageDialog(thrower, e, "Error", JOptionPane.ERROR_MESSAGE);
    }
    
    /**
     * Method used to fail the application if a fatal error is found.
     */
    public static void fail(String message) {
        System.err.println(message);
        System.exit(1);
    }
}