/* ============================================================================
 * Copyright (c) 1998 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Every modification must be notified to the Java Apache Project
 *    and redistribution of the modified code without prior notification
 *    is NOT permitted in any form.
 *
 * 4. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache Project
 *    (http://java.apache.org/)."
 *
 * 5. The names "Apache JMeter" and "Java Apache Project" must not be used to 
 *    endorse or promote products derived from this software without prior 
 *    written permission.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache Project
 *    (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software was developed by Beta Version Productions and 
 * donated to the Java Apache Project that continues its development.
 * For more information on the Java Apache Project and the JMeter project, 
 * please see <http://java.apache.org/>.
 */

package org.apache.jmeter.visualisers;

import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * This class implements a scrolling bar visualiser with 
 * user selectable scale.
 * 
 * This visualiser emulates LiveSoftware ServletKiller.
 *
 * @see org.apache.jmeter.visualisers.Visualiser
 * @author  Stefano Mazzocchi <mazzocch@mbox.systemy.it>
 * @version $Revision: 1.2 $ $Date: 1998/12/14 00:52:11 $
 */
public class BarVisualiser extends JPanel implements Visualiser, ChangeListener {

    private static final int MAX_SCALE = 2000;
    
    private Graph graph;
    private JSlider slider;
    
    class Graph extends JComponent {

        private final int GRIDS = 10;
        
        private int limit;
        private int scale;
        private int counter;
        private Vector samples;
        private Color barColor;
        
        public Graph() {
            super();
            
            this.samples = new Vector();
            this.barColor = Color.lightGray;
            this.counter = 0;
            this.limit = 1000;
            this.scale = limit / GRIDS;
        }

        public synchronized void setLimit(int limit) {
            this.limit = limit;
            this.scale = limit / GRIDS;
        }
        
        public void add(int sample) {
            this.samples.addElement(new Integer(sample));
            this.counter++;
        }
        
        public void clear() {
            this.samples.removeAllElements();
            this.counter = 0;
        }
        
        public void paintComponent(Graphics g) {
            super.paintComponent(g);

            Dimension d = this.getSize();

            for (int i = 1, y; i < GRIDS; i++) {
                y = d.height - (i * d.height / GRIDS);
                g.setColor(Color.black);
                g.drawLine(5, y, d.width - 35, y);
                g.setColor(Color.blue);
                g.drawString(Integer.toString(i * scale), d.width - 30, y + this.getFont().getSize() / 2);
            }

            int bars = (d.width - 40) / 8;
            if (counter < bars) bars = counter;
            
            for (int i = bars; i > 0; i--) {
                int sample = ((Integer) this.samples.elementAt(this.counter - i)).intValue() * d.height / this.limit;
                g.setColor(this.barColor);
                g.fill3DRect(d.width - 36 - 8 * i, d.height - sample, 5, sample, true);
            }
        }
    }
    
    public BarVisualiser() {
        super();
        
        this.graph = new Graph();
        this.slider = new JSlider(JSlider.VERTICAL, 0, MAX_SCALE, 1000);
        this.slider.addChangeListener(this);
        this.slider.setPaintLabels(false);
        this.slider.setPaintTicks(false);
        
        this.setLayout(new BorderLayout());
        this.add(slider, BorderLayout.WEST);
        this.add(graph, BorderLayout.CENTER);
    }    

    public void stateChanged(ChangeEvent e) {
        int limit = this.slider.getValue();
        this.graph.setLimit(limit);
        repaint();
    }
    
    public synchronized void add(long s) {
        this.graph.add((int) s);
        repaint();
    }
    
    public synchronized void clear() {
        this.graph.clear();
        repaint();
    }

    public String toString() {
        return "Scrolling bars visualiser";
    }
}